unit UFormHelp;

{$mode delphi}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls;

type

  { TFormHelp }

  TFormHelp = class(TForm)
    Memo1: TMemo;
    procedure FormCreate(Sender: TObject);
  private
    Output: string;
  public
    procedure AddText(Text: string);
    procedure AddLine(Text: string = '');
  end;

var
  FormHelp: TFormHelp;


implementation

{$R *.lfm}

uses
  UInstructions;

{ TFormHelp }

procedure TFormHelp.FormCreate(Sender: TObject);
var
  InstructionSet: TInstructionSet;
  Instruction: TInstructionInfo;
  I: Integer;
  J: Integer;
begin
  Output := '';
  InstructionSet := TInstructionSet.Create;
  AddLine('Registers:');
  AddLine('R0-R15: There are in total 16 registers available for general use.');
  AddLine('SP (Stack Pointer) - points to the top of stack. It grows downwards.');
  AddLine('IP (Instruction Pointer) - pointer to memory location where current executed instruction is located.');
  AddLine;
  AddLine('Instructions:');
  for I := 0 to InstructionSet.Items.Count - 1 do begin
    Instruction := InstructionSet.Items[I];
    AddText(Instruction.Name);
    for J := 0 to Length(Instruction.Params) - 1 do begin
      if J > 0 then AddText(', ') else AddText(' ');
      if Instruction.Params[J] = ptNumber then AddText('n')
      else if Instruction.Params[J] = ptReg then AddText('R');
    end;
    AddLine;
    AddLine(Instruction.Description);
    AddLine;
  end;

  InstructionSet.Free;
  Memo1.Text := Output;
end;

procedure TFormHelp.AddText(Text: string);
begin
  Output := Output + Text;
end;

procedure TFormHelp.AddLine(Text: string = '');
begin
  AddText(Text + LineEnding);
end;

end.

