unit UCore;

{$mode delphi}

interface

uses
  Classes, SysUtils, ActnList, Controls, Forms, UMachine, UMemory, UMessages,
  UDisassembler, UAssembler;

type

  { TCore }

  TCore = class(TDataModule)
    AHelp: TAction;
    ADisassembler: TAction;
    AConsole: TAction;
    ACpu: TAction;
    AScreen: TAction;
    ARunToCursor: TAction;
    AStop: TAction;
    AStepOver: TAction;
    AStepOut: TAction;
    AStepInto: TAction;
    APause: TAction;
    ARun: TAction;
    ACompile: TAction;
    AExit: TAction;
    ActionList1: TActionList;
    ImageList1: TImageList;
    procedure ACompileExecute(Sender: TObject);
    procedure AConsoleExecute(Sender: TObject);
    procedure ACpuExecute(Sender: TObject);
    procedure ADisassemblerExecute(Sender: TObject);
    procedure AExitExecute(Sender: TObject);
    procedure AHelpExecute(Sender: TObject);
    procedure ARunExecute(Sender: TObject);
    procedure AScreenExecute(Sender: TObject);
    procedure AStopExecute(Sender: TObject);
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private

  public
    Assembler: TAssembler;
    Disassembler: TDisassembler;
    Machine: TMachine;
    CompiledProgram: TMemory;
  end;

var
  Core: TCore;


implementation

{$R *.lfm}

uses
  UFormScreen, UFormCpu, UFormAssembler, UFormConsole, UFormHelp,
  UFormDisassembler, UFormMessages;

{ TCore }

procedure TCore.AExitExecute(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TCore.AHelpExecute(Sender: TObject);
begin
  FormHelp.Show;
end;

procedure TCore.ARunExecute(Sender: TObject);
begin
  ACompile.Execute;
  Machine.PowerOn;
end;

procedure TCore.AScreenExecute(Sender: TObject);
begin
  FormScreen.Screen := Machine.Screen;
  FormScreen.Show;
end;

procedure TCore.AStopExecute(Sender: TObject);
begin
  Machine.PowerOff;
end;

procedure TCore.DataModuleCreate(Sender: TObject);
begin
  Machine := TMachine.Create;
  CompiledProgram := TMemory.Create;
  Assembler := TAssembler.Create;
  Disassembler := TDisassembler.Create;
end;

procedure TCore.DataModuleDestroy(Sender: TObject);
begin
  FreeAndNil(Disassembler);
  FreeAndNil(Assembler);
  FreeAndNil(CompiledProgram);
  FreeAndNil(Machine);
end;

procedure TCore.ACompileExecute(Sender: TObject);
begin
  AStop.Execute;
  with Assembler do begin
    Compile(FormAssembler.SynEdit1.Lines.Text);
    //Memory.SaveToFile('compiled.bin');
    CompiledProgram.Assign(Memory);
    Machine.Memory.Position := 0;
    Machine.Memory.WriteMemory(CompiledProgram);
  end;
  if FormMessages.Visible then FormMessages.Reload;
end;

procedure TCore.AConsoleExecute(Sender: TObject);
begin
  FormConsole.Console := Machine.Console;
  FormConsole.Show;
end;

procedure TCore.ACpuExecute(Sender: TObject);
begin
  FormCpu.Cpu := Machine.Cpu;
  FormCpu.Show;
end;

procedure TCore.ADisassemblerExecute(Sender: TObject);
begin
  with Disassembler do begin
    Memory := CompiledProgram;
    Disassemble(FormDisassembler.MemoCode.Lines);
  end;
  FormDisassembler.Show;
end;

end.

